function evalvbprb(varargin)
% function evalvbprb(ifname, resfname, nkernels, basismdl, basisprec,
% g, h, gamma, maxfolds, Pthrs, refsubsno, itermax, acceps, gpriocnt, dosyscycle)
% evaluates the hierarchical variational bayesian subset evaluation with probit link.
% The algorithm calculates the subset probabilities under the
% probit model.
%
% IMPORTANT NOTE: (checked again 05 2006)
% All probabilities that we find in the results file 
% (subsprobs, indprobs, P_sysmsr, P_sysind and the respective fold \
% estimates) do consider the order of subsets propperly. Irrespective, \
% whether we choose the empty subset or a reference subset, the order
% in the probability matrices correspoinds to the original order in \
% the subsets.
%
% ifname:       name of a definition file that is used to specify this comparison. 
%               ifname structure:
%               fname:    infile_a.mat (containing subsets and train)
%               labels:   1   nan  0   0   1   1   nan (labels that must contain as many entries as 
%                                                       there are rows in train!)
%               fnanem:    infile_b.mat
%               labels:   nan  0   nan  1   1   0   0   1 (etc...)
%               ....
%               each pair generates a separate entry in allsystrain and allsysytest.
%               train [inputs, targets]
%               inputs are arbitrary targets are 1- based class labels.
%               subsets {[coldx11, coldx12,...]; [coldx21, coldx22,...];...}
%               a cell array of subset definitions.
% resfname:     stores indprobs, subsprobs, the probabilities of each subset.
%               if folds > 1 pr we have a testfname, we also predict probabilities for the
%               test data. In this case the result file also contains
%               truetargs (true labels), gtargs (predicted labels) and Pclass, the predicted probabilities.
% nkernels:     number of kernels in basis set definition.
% basismdl:     type of basis functions ('g' for Gaussian and 
%               't' for thin plate spline, 'l' for none (linear))
% basisprec:    precision of basis function used by 'g'.
% g, h:         Gamma prior (i.e. diagonal Wishart) over precision of the prior over weights.
% gamma  :      precision of latent space Gaussian.
% maxfolds:     maximal number of folds. can be fewer if we have not enough data (this is adjusted below).
% Pthrs:        threshold of the cumulative probability. used during prediction
%               to restrict the number of subsets in the predictions to
%               those where the cumulative probability is below. (since the
%               subsets are oprdered w.r.t. decreasing posterior
%               probability this eqals to predict with the most imporatant
%               models. Each models prediction is weighted with its
%               adjusted posterior. If this factor is above 1 all subsets
%               are used.
% refsubsno:    a number into subsets indicating a reference subset.
%               If the value is 'NA', we use the empty subset as reference. 
% itermax:      maximal number of iterations in the vb maximisation
% acceps:       upper tolerance limit of absolute difference between successive
%               neg. free energies before vb maximisation terminates.
% gpriocnts:    a numerical value that specifies prior counts of the beta distribution or a fraction. 
%               then this is a fraction of number of subsets we use as prior counts.
% dosyscycle:   optional parameter: if 1 each fold test is run for every system separately. 
%               This option has only an effect for folds > 1 and will use all systems but 1 as 
%               training data (to infer the common Q(Ig)) and use fold runs over the current system 
%               to get predictions. If this parameter is missing or 0, we do fold tests for all 
%               systems in parallel.
%               
% (C) P. Sykacek 2005

global vbprb;
global g_vals;

fid=fopen('state.mat','r');
if fid ~= -1 % then we set the states from file.
    fclose(fid);
    zrnd=sload('state.mat', 'zrnd');
    rand('state', zrnd);
    zrndnrm=sload('state.mat', 'zrndnrm');
    randn('state', zrndnrm);
end

def_fname=varargin{1};
resfname=varargin{2};
nkernels=str2num(varargin{3});
basismdl=varargin{4};
basisprec=str2num(varargin{5});
g=str2num(varargin{6});
h=str2num(varargin{7});
gamma=str2num(varargin{8});
maxfolds=str2num(varargin{9});
Pthrs=str2num(varargin{10});
refsubs=varargin{11};
if strcmp(refsubs, 'NA')
    refsubs=[];
else
    refsubs=str2num(refsubs);
    if isempty(refsubs)
        refsubs='last';
    end
end
itermax=str2num(varargin{12});
acceps=str2num(varargin{13});
priocnts=str2num(varargin{14});
dosyscycle=str2num(varargin{15});
if nargin >=16
    dosyscycle = str2num(varargin{16});
else
    dosyscycle=0;
end
out_strg=sprintf(['evalvbprb def_fname: %s, resfname: %s, nkernels: %d, basismdl: %s, basisprec: %f,',...
    'g: %f, h: %f, gamma: %f, folds: %d, Pthrs: %f, refsubs: %s, itermax: %d, acceps: %f, priocnts: %f, syscycle: %d'],...
    def_fname, resfname, nkernels, basismdl, basisprec, g, h, gamma, maxfolds, Pthrs, varargin{11}, itermax,...
    acceps, priocnts, dosyscycle);
disp(out_strg);

basisf.btype=basismdl;
basisf.prec=basisprec;
basisf.nr=nkernels;

% we do not remove nan's from the test data. The entire simulation will first
% do the fold results and then provide predictions for the test cases.
[subsets, allsystrain, allsystest, alltrdx, alltsdx]=prep_msys(def_fname, 0);
% randomize data for fold results
if maxfolds > 1
    [allsysfolddat]=allrandomize(allsystrain);
end

% init reference subset if it was a number and
% remove the corresponding subset from subsets.
if strcmp(refsubs, 'last')
    refsubspos=length(subsets);
    refsubs=subsets{refsubspos};
    subsets=subsets(1:(refsubspos-1));
    subsets=[subsets, {[]}];
elseif ~isempty(refsubs)
    refsubspos=refsubs(1);
    refsubs=subsets{refsubspos};
    subsets=subsets([1:(refsubspos-1), (refsubspos+1):length(subsets)]);
    subsets=[subsets, {[]}];
end
if isempty(refsubs) % then we use the empty subset as reference and that is made the last subset later
    refsubspos=length(subsets)+1;
end
    
%subsets
%refsubs
% initialize all inidcator priors
% if nan we use default counts 1
numprival=priocnts;
if isnan(numprival)
    numprival=[1, 1];
end
% if one dimensional and fraction (i.1 between 0 and 1) we reinitialise
% by using fraction times length(subsets) counts as prior.
if length(numprival)==1 & 0 <= numprival & numprival <=1
    numprival=numprival(ones(2,1))*length(subsets);
end
indpriocnts=numprival(:)

% loop over systems (in the fold per systems case!)
nsys=length(allsystrain);
if dosyscycle & maxfolds > 1
    sysno=1;
    % we loop over systems and folds if dosyscycle is 1.
    while sysno <= nsys
        findP=[];
        fsubsP=[];
        fP_sysind=[];
        fP_sysmsr=[];
        fPclss=[];
        fgtrgs=[];
        ftruetargs=[];
        %end
        % first we adjust the folds for that system.
        [folds]=msysiniflds(allsysfolddat, maxfolds, sysno);
        fold=1;
        while folds>=fold
            if folds >1 % we might have got set to 1 (which means that oe of te systems has only one datapoint!)
                [alltran, alltest]=msysnfldvdat(allsysfolddat, folds, fold, sysno);
                havetest=1;
            else
                alltran=allsystrain;
                % alltest={[]};
                % alltest=alltest(ones(1, length(allsystran)));
                havetest=0;
            end
            % init the global objects vbprb and g_vals
            vballgprb(g, h, alltran, basisf, subsets, refsubs,...
                refsubspos, indpriocnts, gamma, itermax, acceps);
            %vbprb.g_mdls(1)
            %return
            % and run maximization (vbprb and g_vals are global)
            %[vbprb]=vbprb_tran(vbprb, g_vals);
            vbprb_tran;
            % extract the gene probabilities to store them
            % first indicator probabilities
            doindP=1;
            skipempty=1;
            indprobs=vbprb_getmdlprbs(doindP, skipempty);
            % we also extract system specific probabilities (for storage).
            [KLsysind, P_sysind, KLsysmsr, P_sysmsr]=vbprb_getsysKL(skipempty);            
            % now posterior probabilities (including null model which is the last one!)
            doindP=0;
            skipempty=0;
            subsprobs=vbprb_getmdlprbs(doindP, skipempty);
            % finally we predict target probabilities if we have test data.
            if havetest
                [Pclass, gtargs, truetargs]=vbprb_pred(alltest, Pthrs);
                fPclss=[fPclss; Pclass{sysno}];
                fgtrgs=[fgtrgs; gtargs{sysno}];
                ftruetargs=[ftruetargs; truetargs{sysno}];
            end
            findP=[findP; indprobs];
            fsubsP=[fsubsP; subsprobs];
            fP_sysind=[fP_sysind; P_sysind(2,:,:)];
            % adjust sysmsr from [nsys x nsubsets] to [1x nsubsets x nsys]
            P_sysmsr(1,:,:)=P_sysmsr';
            fP_sysmsr=[fP_sysmsr; P_sysmsr];
            fold=fold+1;
        end
        foldpred(sysno).findP=findP;
        foldpred(sysno).fsubsP=fsubsP;
        % both .fP_sysind and .fP_sysmsr are [nfold x nsubsets x nsys]
        foldpred(sysno).fP_sysind=fP_sysind;
        foldpred(sysno).fP_sysmsr=fP_sysmsr;
        foldpred(sysno).ftruetargs=ftruetargs;
        foldpred(sysno).fgtrgs=fgtrgs;
        foldpred(sysno).fPclss=fPclss;        
        sysno=sysno+1;
    end
end % we do not cycle over systems and do all folds together
if ~dosyscycle & maxfolds > 1
    % now we adjust maxfolds to the smallest dataset.
    for n=1: nsys
        foldpred(n).findP=[];
        foldpred(n).fsubsP=[];
        foldpred(n).fP_sysind=[];
        foldpred(n).fP_sysmsr=[];
        foldpred(n).ftruetargs=[];
        foldpred(n).fgtrgs=[];
        foldpred(n).fPclss=[];
    end
%     fPclss={};
%     fgtrgs={};
%     ftruetargs={};
%     findP=[];
%     fsubsP=[];
    [folds]=msysiniflds(allsysfolddat, maxfolds)
    fold=1;
    while folds>=fold
        if folds > 1
            [alltran, alltest]=msysnfldvdat(allsysfolddat, folds, fold);
            havetest=1;
            %     tstin=tstdat(:,1:nftrs);
            %     truetargs=tstdat(:,nftrs+1);
        else
            alltran=allsystrain;
            havetest=0;
        end
        % init the object
        %[vbprb, g_vals]=vballgprb(g, h, alltran, basisf, subsets, refsubs, refsubspos, indpriocnts, gamma, itermax, acceps);
        vballgprb(g, h, alltran, basisf, subsets, refsubs, refsubspos, indpriocnts, gamma, itermax, acceps);
        %vbprb.g_mdls(1)
        %return
        % and run maximization
        vbprb_tran;
        % extract the gene probabilities to store them
        % first indicator probabilities
        doindP=1;
        skipempty=1;
        indprobs=vbprb_getmdlprbs(doindP, skipempty);
        % now posterior probabilities (including null model which is the last one!)
        doindP=0;
        skipempty=0;
        subsprobs=vbprb_getmdlprbs(doindP, skipempty);
        % we also extract system specific probabilities (for storage).
        [KLsysind, P_sysind, KLsysmsr, P_sysmsr]=vbprb_getsysKL(skipempty);
        % finally we predict target probabilities if we have test data.
        if havetest
            [Pclass, gtargs, truetargs]=vbprb_pred(alltest, Pthrs);
        else
            Pclass=cell(nsys, 1);
            gtargs=cell(nsys, 1);
            truetargs=cell(nsys, 1);
        end
        % after the folds loop we may now copy the results into the coresponding system store.
        for n=1: nsys
            foldpred(n).findP=[foldpred(n).findP; indprobs];
            foldpred(n).fsubsP=[foldpred(n).fsubsP; subsprobs];
            % both .fP_sysind and .fP_sysmsr are [nfold x nsubsets x nsys]
            foldpred(n).fP_sysind=[foldpred(n).fP_sysind; P_sysind(2,:,:)];
            if n==1
                tmp(1,:,:)=P_sysmsr';
                P_sysmsr=tmp;
            end
            foldpred(n).fP_sysmsr=[foldpred(n).fP_sysmsr; P_sysmsr];
            foldpred(n).ftruetargs=[foldpred(n).ftruetargs; truetargs{n}];
            foldpred(n).fgtrgs=[foldpred(n).fgtrgs; gtargs{n};];
            foldpred(n).fPclss=[foldpred(n).fPclss; Pclass{n}];
        end
        fold=fold+1;
    end
end
% for illustration purpouse we might later need the training labels:
for n=1:length(allsystrain)
    tmpdt=allsystrain{n};
    alltraintarg{n}=tmpdt(:, size(tmpdt, 2));
end
% irrespectiove whether we have fold predictions or not, we now obtain final probability estimates 
% for subsets and eventually predictions if we have test data.
% init the object
%[vbprb, g_vals]=vballgprb(g, h, allsystrain, basisf, subsets, refsubs, refsubspos, indpriocnts, gamma, itermax, acceps);
vballgprb(g, h, allsystrain, basisf, subsets, refsubs, refsubspos, indpriocnts, gamma, itermax, acceps);
% and run maximization
%[vbprb]=vbprb_tran(vbprb, g_vals);
vbprb_tran();
% extract the gene probabilities to store them
% first indicator probabilities
doindP=1;
skipempty=0;
indprobs=vbprb_getmdlprbs(doindP, skipempty);
% now posterior probabilities (including null model which is the last one!)
doindP=0;
skipempty=0;
subsprobs=vbprb_getmdlprbs(doindP, skipempty);
% calculate for all indicator variables, subsets and systems the indicator
% probability and for every subset and system the KL divergence between the
% overall indicator probability and the system specific indicator
% probability.
[KLsysind, P_sysind, KLsysmsr, P_sysmsr]=vbprb_getsysKL(skipempty);
% and finally obtain predictions (all empty preds are handeled internally anyway)
[Pclass, gtargs, truetargs]=vbprb_pred(allsystest, Pthrs);
havefolds=maxfolds>1;
% save all information.
% we save:
% havefolds
% if havefolds == 1 the fold predictions (foldpred) with entries: findP, fsubsP, ftruetargs, fgtrgs, fPclass
% these are the results obtained 
% indprobs and subsprobs (the indicator posteriors and the measure accross subsets).
% vbprb - the final vb object to get info about optimization
% Pclass gtargs and truetargs - predictions and true targets on test data (i.e. the samples 
% labeled nan and the test set if one was specified).
% alltrdx, alltsdx that can be used with training data to obtain the correcot ordering of predictions
% e.g. useful if we predict nan's within a time course) alltsdx are indices in the original dataset. The 
% corresponding predictions are the first length(alltsdx{n}) samples in Pclass{n}.

% ****************
% reminder: indicator probabilities in favoutr of a subset are always at index 2. 
%
% ****************
if havefolds
  save(resfname, 'alltraintarg', 'vbprb', 'subsprobs', 'indprobs', 'KLsysind', 'P_sysind', 'KLsysmsr',...
      'P_sysmsr', 'Pclass', 'truetargs', 'gtargs', 'alltrdx', 'alltsdx', 'havefolds', 'foldpred');
else
  save(resfname, 'alltraintarg', 'vbprb', 'subsprobs', 'indprobs', 'KLsysind', 'P_sysind', 'KLsysmsr',...
      'P_sysmsr', 'Pclass', 'truetargs', 'gtargs', 'alltrdx', 'alltsdx', 'havefolds');
end

% finally we save the state of the random number generators. (to avoid
% deterministic repetitions of the algorithm).
zrnd=rand('state');
zrndnrm=randn('state');
save('state.mat', 'zrnd', 'zrndnrm');